package cd.casic.ci.process.api.process.utils;

import cd.casic.ci.common.pipeline.constant.StringConstant;
import cd.casic.ci.common.pipeline.enums.BuildFormPropertyType;
import cd.casic.ci.common.pipeline.pojo.BuildParameters;
import cn.hutool.core.lang.Pair;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.regex.Pattern;

/**
 * @author by mianbin
 * @Classname PipelineVarUtil
 * @Description TODO
 * @Date 2025/4/1 16:52
 */
@Slf4j
@Component
public class PipelineVarUtil {

    private static final Pattern tPattern = Pattern.compile("\\$[{]{2}(?<double>[^$^{}]+)[}]{2}");
    public static final String CONTEXT_PREFIX = "variables.";
    public static final int MAX_VERSION_LEN = 64;

    // 前置拼接映射
    private static final Map<String, String> oldPrefixMappingNew = new HashMap<>();
    private static final Map<String, String> newPrefixMappingOld = new HashMap<>();

    // 旧变量到新变量的映射
    private static final Map<String, String> oldVarMappingNewVar = new HashMap<>();
    private static final Map<String, String> newVarMappingOldVar = new HashMap<>();

    // CI 预置上下文转换映射关系
    private static final Map<String, String> contextVarMappingBuildVar = new HashMap<>();
    private static final Map<String, String> contextVarMappingBuildVarRevert = new HashMap<>();
    private static final Map<String, String> reverseContextVarMappingBuildVar = new HashMap<>();

    static {
        // 初始化 oldPrefixMappingNew
        oldPrefixMappingNew.put("pipeline.material.url", "PIPELINE_MATERIAL_URL");
        oldPrefixMappingNew.put("pipeline.material.branchName", "PIPELINE_MATERIAL_BRANCHNAME");
        // 其他旧前缀到新前缀的映射...

        // 初始化 newPrefixMappingOld
        for (Map.Entry<String, String> entry : oldPrefixMappingNew.entrySet()) {
            newPrefixMappingOld.put(entry.getValue(), entry.getKey());
        }

        // 初始化 oldVarMappingNewVar
        oldVarMappingNewVar.put("pipeline.start.isMobile", "PIPELINE_START_MOBILE");
        oldVarMappingNewVar.put("repoName", "PIPELINE_REPO_NAME");
        // 其他旧变量到新变量的映射...

        // 初始化 newVarMappingOldVar
        for (Map.Entry<String, String> entry : oldVarMappingNewVar.entrySet()) {
            newVarMappingOldVar.put(entry.getValue(), entry.getKey());
        }

        // 初始化 contextVarMappingBuildVar
        contextVarMappingBuildVar.put("ci.project_name", "PROJECT_NAME_CHINESE");
        contextVarMappingBuildVar.put("ci.build_msg", "PIPELINE_BUILD_MSG");
        // 其他 CI 预置上下文到构建变量的映射...

        // 初始化 contextVarMappingBuildVarRevert
        contextVarMappingBuildVarRevert.put("PIPELINE_START_USER_NAME", "ci.actor");

        // 初始化 reverseContextVarMappingBuildVar
        for (Map.Entry<String, String> entry : contextVarMappingBuildVar.entrySet()) {
            reverseContextVarMappingBuildVar.put(entry.getValue(), entry.getKey());
        }
    }

    /**
     * 检查[keyword]字符串是不是一个变量语法， ${{ varName }}， 如果不是则返回false
     * 注意：已不再支持 ${ var } 旧的语法定义变量，只支持全新 ${{ var }} 的语法
     */
    public static boolean isVar(String keyword) {
        return keyword != null && !keyword.isEmpty() && tPattern.matcher(keyword).matches();
    }

    /**
     * 检查[keyword]串中有没有变量，比如  "abc_${{ varName }} is true" 将识别出存在 varName变量，会返回true
     */
    public boolean haveVar(String keyword) {
        return tPattern.matcher(keyword).find();
    }

    /**
     * 获取反向变量名
     */
    public String fetchReverseVarName(String contextKey) {
        Map<String, String> varMap = new HashMap<>(reverseContextVarMappingBuildVar);
        varMap.putAll(contextVarMappingBuildVarRevert);
        return varMap.get(contextKey);
    }

    /**
     * 填充CI预置变量
     */
    public static void fillContextVarMap(Map<String, String> varMap, Map<String, String> buildVar) {
        for (Map.Entry<String, String> entry : contextVarMappingBuildVar.entrySet()) {
            String varKey = entry.getValue();
            if (buildVar.containsKey(varKey) && buildVar.get(varKey) != null && !buildVar.get(varKey).isEmpty()) {
                varMap.put(entry.getKey(), buildVar.get(varKey));
            }
        }
    }

    /**
     * 填充CI预置变量
     */
    public static Map<String, String> fillContextVarMap(Map<String, String> buildVar) {
        Map<String, String> varMap = new HashMap<>();
        fillContextVarMap(varMap, buildVar);
        varMap.putAll(buildVar);
        return varMap;
    }

    /**
     * 获取CI预置变量
     */
    public String fetchContextInBuildVars(String contextKey, Map<String, String> buildVar) {
        String varKey = contextVarMappingBuildVar.get(contextKey);
        return buildVar.get(varKey);
    }

    /**
     * 获取CI预置变量名
     */
    public String fetchVarName(String contextKey) {
        return contextVarMappingBuildVar.get(contextKey);
    }

    /**
     * 填充旧变量名，兼容用户在流水线中旧的写法
     */
    public void fillOldVarWithType(Map<String, Pair<String, BuildFormPropertyType>> varMaps) {
        turningWithType(newVarMappingOldVar, varMaps, false);
        prefixTurningWithType(newPrefixMappingOld, varMaps, false);
    }

    /**
     * 填充旧变量名，兼容用户在流水线中旧的写法
     */
    public void fillOldVar(Map<String, String> vars) {
        turning(newVarMappingOldVar, vars, false);
        prefixTurning(newPrefixMappingOld, vars, false);
    }

    /**
     * 填充variable变量
     */
    public Map<String, String> fillVariableMap(Map<String, String> pipelineParamMap) {
        Map<String, String> allVars = new HashMap<>();
        for (Map.Entry<String, String> entry : pipelineParamMap.entrySet()) {
            String name = entry.getKey();
            String value = entry.getValue();
            allVars.put(name, value);
            if (!name.startsWith(CONTEXT_PREFIX)) {
                allVars.put(CONTEXT_PREFIX + name, value);
            }
        }
        return allVars;
    }

    /**
     * 从新变量前缀的变量中查出并增加旧变量，会做去重
     */
    public Map<String, String> mixOldVarAndNewVar(Map<String, String> vars) {
        prefixTurning(newPrefixMappingOld, vars, false);
        Map<String, String> allVars = new HashMap<>();
        for (Map.Entry<String, String> entry : vars.entrySet()) {
            String key = entry.getKey();
            String value = entry.getValue();
            String oldVarName = newVarToOldVar(key);
            if (oldVarName != null && !oldVarName.isEmpty()) {
                allVars.put(oldVarName, value);
                allVars.put(key, value);
            } else {
                String newVarName = oldVarToNewVar(key);
                if (newVarName != null && !newVarName.isEmpty() && !vars.containsKey(newVarName)) {
                    allVars.put(newVarName, value);
                }
                if (!allVars.containsKey(key) || "BuildNo".equals(key)) {
                    allVars.put(key, value);
                }
            }
        }
        return allVars;
    }

    /**
     * 将[varMaps]找到旧变量并替换成新变量，并删除掉旧变量
     */
    public void replaceOldByNewVar(Map<String, Pair<String, BuildFormPropertyType>> varMaps) {
        turningWithType(oldVarMappingNewVar, varMaps, true);
        prefixTurningWithType(oldPrefixMappingNew, varMaps, true);
    }

    /**
     * 遍历[varMaps]中的fullKey，与[mapping]中的key做完成匹配，匹配成功后从[mapping]取出value作newKey
     * 并写入到[varMaps],最后根据[replace]值为true决定要删除[varMaps]中的fullKey
     */
    private void turningWithType(Map<String, String> mapping, Map<String, Pair<String, BuildFormPropertyType>> varMaps, boolean replace) {
        for (Map.Entry<String, String> entry : mapping.entrySet()) {
            String oldKey = entry.getKey();
            String newKey = entry.getValue();
            if (varMaps.containsKey(oldKey) && !varMaps.containsKey(newKey)) {
                varMaps.put(newKey, varMaps.get(oldKey));
                if (replace && !"BuildNo".equals(oldKey)) {
                    varMaps.remove(oldKey);
                }
            }
        }
    }

    /**
     * 遍历[vars]中的fullKey，与[mapping]中的key做完成匹配，匹配成功后从[mapping]取出value作newKey
     * 并写入到[vars]，最后根据[replace]值为true决定要删除[vars]中的fullKey
     */
    private void turning(Map<String, String> mapping, Map<String, String> vars, boolean replace) {
        for (Map.Entry<String, String> entry : mapping.entrySet()) {
            String oldKey = entry.getKey();
            String newKey = entry.getValue();
            if (vars.containsKey(oldKey) && !vars.containsKey(newKey)) {
                vars.put(newKey, vars.get(oldKey));
                if (replace) {
                    vars.remove(oldKey);
                }
            }
        }
    }

    /**
     * 遍历[vars]中的fullKey，与[mapping]中的key做前缀匹配，匹配成功后从[mapping]取出value,替换掉fullKey的前缀key，组成newKey
     * 并写入到[vars],最后根据[replace]值为true决定要删除[vars]中的fullKey
     */
    private void prefixTurning(Map<String, String> mapping, Map<String, String> vars, boolean replace) {
        Set<String> keys = new HashSet<>(vars.keySet());
        for (String fullKey : keys) {
            for (Map.Entry<String, String> entry : mapping.entrySet()) {
                String key = entry.getKey();
                String value = entry.getValue();
                if (fullKey.startsWith(key)) {
                    String newKey = value + fullKey.substring(key.length());
                    vars.put(newKey, vars.get(fullKey));
                    if (replace) {
                        vars.remove(fullKey);
                    }
                    break;
                }
            }
        }
    }

    /**
     * 遍历[varMaps]中的fullKey，与[mapping]中的key做前缀匹配，匹配成功后从[mapping]取出value,替换掉fullKey的前缀key，组成newKey
     * 并写入到[varMaps],最后根据[replace]值为true决定要删除[varMaps]中的fullKey
     */
    private void prefixTurningWithType(Map<String, String> mapping, Map<String, Pair<String, BuildFormPropertyType>> varMaps, boolean replace) {
        Set<String> keys = new HashSet<>(varMaps.keySet());
        for (String fullKey : keys) {
            for (Map.Entry<String, String> entry : mapping.entrySet()) {
                String key = entry.getKey();
                String value = entry.getValue();
                if (fullKey.startsWith(key)) {
                    String newKey = value + fullKey.substring(key.length());
                    varMaps.put(newKey, varMaps.get(fullKey));
                    if (replace) {
                        varMaps.remove(fullKey);
                    }
                    break;
                }
            }
        }
    }

    public static String oldVarToNewVar(String oldVarName) {
        return oldVarMappingNewVar.get(oldVarName);
    }

    public String newVarToOldVar(String newVarName) {
        return newVarMappingOldVar.get(newVarName);
    }

    public Map<String, String> contextVarMap() {
        return contextVarMappingBuildVar;
    }

    /**
     * 从流水线启动参数[buildParameters]中找出推荐版本号,由[MAJORVERSION].[MINORVERSION].[FIXVERSION].[BUILD_NO]组成
     * 需要注意的是，旧的参数命名继续兼容有效，由{MarjorVersion}.{MinorVersion}.{FixVersion}.{BuildNo} 组成，所以启动参数切记
     * 不要与此命名相同造成了冲突
     */
    public String getRecommendVersion(List<BuildParameters> buildParameters) {
        String recommendVersionPrefix = getRecommendVersionPrefix(buildParameters);
        if (recommendVersionPrefix == null) {
            return null;
        }
        String buildNo = null;
        for (BuildParameters param : buildParameters) {
            if ("BUILD_NO".equals(param.getKey()) || "BuildNo".equals(param.getKey())) {
                buildNo = param.getValue() != null ? param.getValue().toString() : null;
                break;
            }
        }
        if (buildNo == null) {
            return null;
        }
        return StringConstant.coerceAtMaxLength(recommendVersionPrefix + "." + buildNo, MAX_VERSION_LEN);
    }

    /**
     * 从流水线启动参数[buildParameters]中找出版本号前缀，由[MAJORVERSION].[MINORVERSION].[FIXVERSION] 组成，
     * 如果[buildParameters]中不存在上述3类参数，则返回空
     * 需要注意的是，旧的参数命名继续兼容有效，由{MarjorVersion}.{MinorVersion}.{FixVersion} 组成，所以启动参数切记
     * 不要与此命名相同造成了冲突
     */
    public String getRecommendVersionPrefix(List<BuildParameters> buildParameters) {
        String majorVersion = null;
        String minorVersion = null;
        String fixVersion = null;
        for (BuildParameters param : buildParameters) {
            if ("MAJORVERSION".equals(param.getKey()) || "MajorVersion".equals(param.getKey())) {
                majorVersion = param.getValue() != null ? param.getValue().toString() : null;
            } else if ("MINORVERSION".equals(param.getKey()) || "MinorVersion".equals(param.getKey())) {
                minorVersion = param.getValue() != null ? param.getValue().toString() : null;
            } else if ("FIXVERSION".equals(param.getKey()) || "FixVersion".equals(param.getKey())) {
                fixVersion = param.getValue() != null ? param.getValue().toString() : null;
            }
        }
        if (majorVersion == null || minorVersion == null || fixVersion == null) {
            return null;
        }
        return majorVersion + "." + minorVersion + "." + fixVersion;
    }

    public boolean recommendVersionKey(String key) {
        return "MAJORVERSION".equals(key) || "MINORVERSION".equals(key) || "FIXVERSION".equals(key);
    }
}