package cd.casic.pipeline.store.api.image;


import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.pipeline.store.pojo.common.InstalledProjRespItem;
import cd.casic.pipeline.store.pojo.image.enums.ImageAgentTypeEnum;
import cd.casic.pipeline.store.pojo.image.enums.ImageRDTypeEnum;
import cd.casic.pipeline.store.pojo.image.request.InstallImageReq;
import cd.casic.pipeline.store.pojo.image.response.JobImageItem;
import cd.casic.pipeline.store.pojo.image.response.JobMarketImageItem;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;


@Tag(name = "USER_MARKET_IMAGE_PROJECT", description = "研发商店-镜像项目间关系")
@RestController
@RequestMapping("/user/market")
public interface UserImageProjectResource {

    @Operation(summary = "安装镜像到项目")
    @PostMapping("/image/install")
    CommonResult<Boolean> installImage(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "安装镜像到项目请求报文体", required = true)
            InstallImageReq installImageReq
    );

    @Operation(summary = "根据镜像标识获取已安装的项目列表")
    @GetMapping("/image/installedProjects/{imageCode}")
    CommonResult<List<InstalledProjRespItem>> getInstalledProjects(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "模版代码", required = true)
            @PathVariable("imageCode")
            String imageCode
    );

    @Operation(summary = "根据项目标识获取可用镜像列表（公共+已安装）")
    @GetMapping("/image/availableImages")
    PageResult<JobImageItem> getAvailableImagesByProjectCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目标识", required = true)
            @RequestParam("projectCode")
            String projectCode,
            @Parameter(description = "机器类型", required = false)
            @RequestParam("agentType")
            ImageAgentTypeEnum agentType,
            @Parameter(description = "是否推荐", required = false)
            @RequestParam("recommendFlag")
            Boolean recommendFlag,
            @Parameter(description = "分类ID", required = false)
            @RequestParam("classifyId")
            String classifyId,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "根据项目标识获取商店镜像列表")
    @GetMapping("/image/jobMarketImages")
    PageResult<JobMarketImageItem> getJobMarketImagesByProjectCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目标识", required = true)
            @RequestParam("projectCode")
            String projectCode,
            @Parameter(description = "机器类型", required = false)
            @RequestParam("agentType")
            ImageAgentTypeEnum agentType,
            @Parameter(description = "是否推荐", required = false)
            @RequestParam("recommendFlag")
            Boolean recommendFlag,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "根据项目标识与镜像名称模糊搜索商店镜像列表（已安装+未安装）")
    @PostMapping("/image/jobMarketImages/search")
    PageResult<JobMarketImageItem> searchJobMarketImages(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目标识", required = true)
            @RequestParam("projectCode")
            String projectCode,
            @Parameter(description = "机器类型", required = false)
            @RequestParam("agentType")
            ImageAgentTypeEnum agentType,
            @Parameter(description = "是否推荐", required = false)
            @RequestParam("recommendFlag")
            Boolean recommendFlag,
            @Parameter(description = "搜索关键字", required = false)
            @RequestParam("keyword")
            String keyword,
            @Parameter(description = "镜像分类Id", required = false)
            @RequestParam("classifyId")
            String classifyId,
            @Parameter(description = "应用范畴", required = false)
            @RequestParam("categoryCode")
            String categoryCode,
            @Parameter(description = "研发来源", required = false)
            @RequestParam("rdType")
            ImageRDTypeEnum rdType,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );
}