package cd.casic.server.controller;

import cd.casic.ci.common.pipeline.req.task.TaskUpdateReq;
import cd.casic.ci.common.pipeline.resp.task.TasksResp;
import cd.casic.ci.process.process.dataObject.log.PipTaskLog;
import cd.casic.ci.process.process.service.task.TaskService;
import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.util.network.IpUtil;
import cd.casic.framework.commons.util.redis.core.utils.IPUtils;
import cd.casic.framework.commons.util.util.WebFrameworkUtils;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.constraints.NotEmpty;
import jakarta.validation.constraints.NotNull;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;

import java.io.IOException;

@RestController
@RequestMapping("/task")
public class TasksController {
    @Resource
    private TaskService taskService;
    /**
     * @pi.name:查询任务及任务详情
     * @pi.path:/tasks/findOneTasksOrTask
     * @pi.methodType:post
     * @pi.request-type: formdata
     * @pi.param: name=taskId;dataType=string;value=taskId;
     */
    @PostMapping(path="/findOneTasksOrTask/{taskId}")
    public CommonResult<TasksResp> findOneTasksOrTask(@NotNull @PathVariable String taskId){
        return CommonResult.success(taskService.getRespById(taskId));
    }
    /**
     *  复制task节点
     * */
    @PostMapping(path="/copyTask/{taskId}")
    public CommonResult<Void> copyTask(@NotEmpty @PathVariable String taskId){
        taskService.copyTask(taskId);
        return CommonResult.success();
    }
    @PostMapping(path="/updateTask")
    public CommonResult<Boolean> updateTask(@RequestBody TaskUpdateReq req){
        Boolean b = taskService.updateTask(req);
        return CommonResult.success(b);
    }
    @PostMapping("/getLogContentByTaskId/{taskId}")
    public CommonResult<String> getLogContentByTaskId(@PathVariable String taskId){
        return taskService.getLogContentByTaskId(taskId);
    }

}
